function watch(target) {
  var subscribers = [];
  var subscribable = new Proxy(target, {
    get: function get(obj, prop) {
      var action = methods[prop];
      if (action) return action;
      return obj[prop];
    },
    set: function set(obj, prop, value) {
      var action = methods[prop];
      if (action) return false;
      obj[prop] = value;
      notifyChangeFor(prop);
      return true;
    }
  });
  var methods = {
    subscribe: function subscribe(cb, subscription) {
      subscribers.push({
        cb: cb,
        subscription: subscription
      });
      return function () {
        return methods.unsubscribe(cb);
      };
    },
    unsubscribe: function unsubscribe(removeCB) {
      var index = subscribers.findIndex(function (_ref) {
        var cb = _ref.cb;
        return cb === removeCB;
      });
      subscribers.splice(index, 1);
    }
  };

  function notifyChangeFor(prop) {
    subscribers.forEach(function (_ref2) {
      var subscription = _ref2.subscription,
          cb = _ref2.cb;

      if (!subscription || subscription[prop]) {
        cb(subscribable, prop);
      }
    });
  }

  return subscribable;
}

function createCollection(entries) {
  if (entries === void 0) {
    entries = {};
  }

  var subscribers = [];
  var collectionMethods = {
    subscribe: function subscribe(listener) {
      subscribers.push({
        listener: listener
      });
      return function () {
        return collectionMethods.unsubscribe(listener);
      };
    },
    unsubscribe: function unsubscribe(listener) {
      var index = subscribers.findIndex(function (sub) {
        return sub.listener === listener;
      });
      subscribers.splice(index, 1);
    },
    notifyChange: function notifyChange(change) {
      subscribers.forEach(function (_ref) {
        var listener = _ref.listener;
        listener(watchableCollection, change);
      });
    },
    toArray: function toArray() {
      return Object.keys(watchableCollection).map(function (id) {
        return watchableCollection[id];
      });
    }
  };
  var watchableCollection = new Proxy(entries, {
    get: function get(obj, prop) {
      var action = collectionMethods[prop];

      if (action) {
        return action;
      }

      return obj[prop];
    },
    set: function set(obj, prop, value) {
      var action = collectionMethods[prop];

      if (action) {
        return false;
      }

      obj[prop] = value;
      collectionMethods.notifyChange({
        change: 'set',
        entry: value,
        id: prop
      });
      return true;
    },
    deleteProperty: function deleteProperty(obj, prop) {
      var action = collectionMethods[prop];

      if (action) {
        return false;
      }

      var entry = obj[prop];
      delete obj[prop];
      collectionMethods.notifyChange({
        change: 'delete',
        entry: entry,
        id: prop
      });
      return true;
    }
  });
  return watchableCollection;
}

export { createCollection, watch };
//# sourceMappingURL=babas.esm.js.map
